"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Scope = exports.modeDescriptionToMode = void 0;
const pattern_1 = require("./pattern");
function modeDescriptionToMode(md) {
    var _a;
    function translateModeOrPattern(m) {
        if (m instanceof pattern_1.PatternBase)
            return m;
        return modeDescriptionToMode(m);
    }
    if ('match' in md)
        return new MatchMode(md['match'], md['name'], md['scope']);
    if ('begin' in md)
        return new SurroundMode(md['begin'], md['end'], (_a = md['contains']) === null || _a === void 0 ? void 0 : _a.map(translateModeOrPattern), md['name'], md['scope']);
    if ('reference' in md)
        return new ReferenceMode(md['reference']);
    if ('contains' in md)
        return new GroupMode(md['contains'].map(translateModeOrPattern), md['name']);
    throw new Error('Unrecognized mode description!');
}
exports.modeDescriptionToMode = modeDescriptionToMode;
/**
 * Represents a mode that consists of a single matched pattern.
 */
class MatchMode {
    /**
     * @param match The pattern to be matched.
     * @param name The name of the mode.
     * @param scope The assigned scope of the mode.
     */
    constructor(match, name, scope) {
        this.match = match;
        this.name = name;
        this.scope = scope;
    }
    toTextMate(language) {
        // TODO: Handle capture groups
        return {
            name: scopeToTextMate(language, this.scope),
            match: this.match.toRegex().regex,
        };
    }
}
/**
 * Represents a mode that is surrounded by two patterns.
 */
class SurroundMode {
    /**
     * @param begin The pattern that starts the mode.
     * @param end The pattern that ends the mode.
     * @param contains The modes or patterns that are contained within this one.
     * @param name The name of the mode.
     * @param scope The assigned scope of the mode.
     */
    constructor(begin, end, contains, name, scope) {
        this.begin = begin;
        this.end = end;
        this.contains = contains;
        this.name = name;
        this.scope = scope;
    }
    toTextMate(language) {
        var _a;
        // TODO: Handle capture groups for begin and end
        return {
            name: scopeToTextMate(language, this.scope),
            begin: this.begin.toRegex().regex,
            end: this.end.toRegex().regex,
            patterns: (_a = this.contains) === null || _a === void 0 ? void 0 : _a.map(m => modeOrPatternToTextMate(language, m)),
        };
    }
}
/**
 * A mode that groups together other modes or patterns.
 */
class GroupMode {
    /**
     * @param contains The modes or patterns that are contained within this one.
     * @param name The name of the mode.
     */
    constructor(contains, name) {
        this.contains = contains;
        this.name = name;
    }
    toTextMate(language) {
        throw new Error('Method not implemented. (group)');
    }
}
/**
 * A mode that references another mode.
 */
class ReferenceMode {
    /**
     * @param reference The referenced mode name.
     */
    constructor(reference) {
        this.reference = reference;
    }
    toTextMate(language) {
        return { include: `#${this.reference}` };
    }
}
/**
 * The possible scopes a liglighted element can have.
 */
var Scope;
(function (Scope) {
    /**
     * A single-line comment.
     */
    Scope[Scope["LineComment"] = 0] = "LineComment";
    /**
     * A multi-line comment.
     */
    Scope[Scope["BlockComment"] = 1] = "BlockComment";
    /**
     * A tag inside a comment, like '@param' or 'FIXME'.
     */
    Scope[Scope["DocTag"] = 2] = "DocTag";
})(Scope = exports.Scope || (exports.Scope = {}));
function scopeToTextMate(lang, scope) {
    function nameLookup(scope) {
        switch (scope) {
            case Scope.LineComment: return "comment.line";
            case Scope.BlockComment: return "comment.block";
            case Scope.DocTag: return "comment.block.documentation";
            default: return undefined;
        }
    }
    let name = nameLookup(scope);
    if (!name)
        return undefined;
    return `${name}.${lang.scopeName}`;
}
function modeOrPatternToTextMate(lang, mp) {
    if (mp instanceof pattern_1.PatternBase)
        return new MatchMode(mp).toTextMate(lang);
    return mp.toTextMate(lang);
}
