/**
 * Constructs a regex pattern from a regex string.
 * @param r The regex string.
 * @returns The constructed regex pattern.
 */
export declare function regex(r: string): IPattern;
/**
 * Constructs a literal pattern from a string.
 * @param l The literal string.
 * @returns The constructed literal pattern.
 */
export declare function literal(l: string): IPattern;
/**
 * Concatenates patterns into a sequence.
 * @param ps The patterns to concatenate.
 * @returns The concatenated pattern.
 */
export declare function cat(...ps: IPattern[]): IPattern;
/**
 * Concatenates patterns into an alternation.
 * @param ps The patterns to concatenate.
 * @returns The concatenated pattern.
 */
export declare function or(...ps: IPattern[]): IPattern;
/**
 * Constructs a repetition pattern.
 * @param p The pattern to repeat.
 * @param min The minimum number of repetitions.
 * @param max The maximum number of repetitions.
 * @returns The constructed repetition pattern.
 */
export declare function rep(p: IPattern, min: number, max?: number): IPattern;
/**
 * Shorthand for @see rep.
 */
export declare function rep0(...ps: IPattern[]): IPattern;
/**
 * Shorthand for @see rep.
 */
export declare function rep1(...ps: IPattern[]): IPattern;
/**
 * Shorthand for @see rep.
 */
export declare function opt(...ps: IPattern[]): IPattern;
/**
 * Shorthand for @see rep.
 */
export declare function atMost(p: IPattern, max: number): IPattern;
/**
 * Shorthand for @see rep.
 */
export declare function atLeast(p: IPattern, min: number): IPattern;
/**
 * Shorthand for @see rep.
 */
export declare function between(p: IPattern, min: number, max: number): IPattern;
/**
 * Shorthand for @see rep.
 */
export declare function exactly(p: IPattern, n: number): IPattern;
/**
 * Captures a pattern with a name.
 * @param ps The pattern to capture.
 * @param name The name to give to the capture.
 * @returns The capture pattern.
 */
export declare function capture(p: IPattern, name: string): IPattern;
/**
 * Creates a lookahead pattern.
 * @param p The pattern to consider lookahead.
 * @param negate True, if the pattern should be negated.
 * @returns The constructed lookahead pattern.
 */
export declare function lookahead(p: IPattern, negate?: boolean): IPattern;
/**
 * Creates a lookbehind pattern.
 * @param p The pattern to consider lookbehind.
 * @param negate True, if the pattern should be negated.
 * @returns The constructed lookbehind pattern.
 */
export declare function lookbehind(p: IPattern, negate?: boolean): IPattern;
/**
 * Appends tags onto a pattern.
 * @param p The pattern to append the tags onto.
 * @param tags The tags to append.
 */
export declare function tag(p: IPattern, ...tags: any[]): IPattern;
/**
 * Represents a pattern that can be translated to a regular expression.
 */
export interface IPattern {
    /**
     * Translates this pattern to a regular expression.
     */
    toRegex(): RegexResult;
    /**
     * Retrieves the underlying pattern, in case this is a metadata element with no
     * contrinution to the regex output.
     */
    contentElement(): IPattern;
    /**
     * Alias for @see capture.
     */
    capture(name: string): IPattern;
    /**
     * Alias for @see tag.
     */
    tag(...tags: any[]): IPattern;
}
/**
 * The result of building a regex.
 */
declare type RegexResult = {
    /**
     * The built regex.
     */
    regex: string;
    /**
     * The precedence of the regex construct returned.
     */
    precedence: number;
    /**
     * The total number of capture groups, including the unnamed ones.
     */
    captureGroupCount: number;
    /**
     * The named capture groups mapped from name to the corresponding pattern.
     */
    captureNames: Map<string, IPattern>;
    /**
     * The capture groups mapped from the pattern to the group number.
     */
    captureGroups: Map<IPattern, number>;
    /**
     * The tags appended to patterns.
     */
    tags: Map<IPattern, any[]>;
};
export declare abstract class PatternBase implements IPattern {
    constructor();
    abstract toRegex(): RegexResult;
    abstract contentElement(): IPattern;
    capture(name: string): IPattern;
    tag(...tags: any[]): IPattern;
}
/**
 * Regex precedences.
 */
export declare namespace Precedence {
    const LOWEST = 0;
    const ALT = 0;
    const SEQ = 1;
    const REP = 2;
    const GROUP = 3;
}
/**
 * Statistics of a regex.
 */
declare type RegexStats = {
    /**
     * The precedence level.
     */
    precedence: number;
    /**
     * The number of capture groups.
     */
    captureGroupCount: number;
};
/**
 * Computes the statistics of a regex.
 * @param regex The regex to compute the stats for.
 * @returns The statistics for the regex.
 */
export declare function regexStats(regex: string): RegexStats;
export {};
