"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.regexStats = exports.Precedence = exports.PatternBase = exports.tag = exports.lookbehind = exports.lookahead = exports.capture = exports.exactly = exports.between = exports.atLeast = exports.atMost = exports.opt = exports.rep1 = exports.rep0 = exports.rep = exports.or = exports.cat = exports.literal = exports.regex = void 0;
/**
 * Constructs a regex pattern from a regex string.
 * @param r The regex string.
 * @returns The constructed regex pattern.
 */
function regex(r) {
    return new RegexPattern(r);
}
exports.regex = regex;
/**
 * Constructs a literal pattern from a string.
 * @param l The literal string.
 * @returns The constructed literal pattern.
 */
function literal(l) {
    return new RegexPattern(escapeRegex(l));
}
exports.literal = literal;
/**
 * Concatenates patterns into a sequence.
 * @param ps The patterns to concatenate.
 * @returns The concatenated pattern.
 */
function cat(...ps) {
    return ps.reduce((acc, p) => new SeqPattern(acc, p));
}
exports.cat = cat;
/**
 * Concatenates patterns into an alternation.
 * @param ps The patterns to concatenate.
 * @returns The concatenated pattern.
 */
function or(...ps) {
    return ps.reduce((acc, p) => new AltPattern(acc, p));
}
exports.or = or;
/**
 * Constructs a repetition pattern.
 * @param p The pattern to repeat.
 * @param min The minimum number of repetitions.
 * @param max The maximum number of repetitions.
 * @returns The constructed repetition pattern.
 */
function rep(p, min, max) {
    return new RepPattern(p, min, max);
}
exports.rep = rep;
/**
 * Shorthand for @see rep.
 */
function rep0(...ps) {
    return rep(cat(...ps), 0, undefined);
}
exports.rep0 = rep0;
/**
 * Shorthand for @see rep.
 */
function rep1(...ps) {
    return rep(cat(...ps), 1, undefined);
}
exports.rep1 = rep1;
/**
 * Shorthand for @see rep.
 */
function opt(...ps) {
    return rep(cat(...ps), 0, 1);
}
exports.opt = opt;
/**
 * Shorthand for @see rep.
 */
function atMost(p, max) {
    return rep(p, 0, max);
}
exports.atMost = atMost;
/**
 * Shorthand for @see rep.
 */
function atLeast(p, min) {
    return rep(p, min, undefined);
}
exports.atLeast = atLeast;
/**
 * Shorthand for @see rep.
 */
function between(p, min, max) {
    return rep(p, min, max);
}
exports.between = between;
/**
 * Shorthand for @see rep.
 */
function exactly(p, n) {
    return rep(p, n, n);
}
exports.exactly = exactly;
/**
 * Captures a pattern with a name.
 * @param ps The pattern to capture.
 * @param name The name to give to the capture.
 * @returns The capture pattern.
 */
function capture(p, name) {
    return new CapturePattern(p, name);
}
exports.capture = capture;
/**
 * Creates a lookahead pattern.
 * @param p The pattern to consider lookahead.
 * @param negate True, if the pattern should be negated.
 * @returns The constructed lookahead pattern.
 */
function lookahead(p, negate = false) {
    return new LookaroundPattern(p, false, negate);
}
exports.lookahead = lookahead;
/**
 * Creates a lookbehind pattern.
 * @param p The pattern to consider lookbehind.
 * @param negate True, if the pattern should be negated.
 * @returns The constructed lookbehind pattern.
 */
function lookbehind(p, negate = false) {
    return new LookaroundPattern(p, true, negate);
}
exports.lookbehind = lookbehind;
var captureCnt = 0;
/**
 * Appends tags onto a pattern.
 * @param p The pattern to append the tags onto.
 * @param tags The tags to append.
 */
function tag(p, ...tags) {
    // Capture the pattern, if needed
    if (!(p instanceof CapturePattern))
        p = capture(p, `tagCapture_${captureCnt++}`);
    return new TagPattern(p, tags);
}
exports.tag = tag;
class PatternBase {
    constructor() { }
    capture(name) {
        return capture(this, name);
    }
    tag(...tags) {
        return tag(this, ...tags);
    }
}
exports.PatternBase = PatternBase;
class RegexPattern extends PatternBase {
    constructor(text) {
        super();
        this.text = text;
    }
    toRegex() {
        let stats = regexStats(this.text);
        return {
            regex: this.text,
            precedence: stats.precedence,
            captureGroupCount: stats.captureGroupCount,
            captureNames: new Map(),
            captureGroups: new Map(),
            tags: new Map(),
        };
    }
    contentElement() {
        return this;
    }
}
class AltPattern extends PatternBase {
    constructor(first, second) {
        super();
        this.first = first;
        this.second = second;
    }
    toRegex() {
        let a = groupForPrecedence(this.first.toRegex(), Precedence.ALT);
        let b = groupForPrecedence(this.second.toRegex(), Precedence.ALT);
        return {
            regex: `${a.regex}|${b.regex}`,
            precedence: Precedence.ALT,
            captureGroupCount: a.captureGroupCount + b.captureGroupCount,
            captureNames: mergeMaps(a.captureNames, b.captureNames),
            captureGroups: mergeMaps(a.captureGroups, shiftCaptureGroups(b.captureGroups, a.captureGroupCount)),
            tags: mergeMaps(a.tags, b.tags),
        };
    }
    contentElement() {
        return this;
    }
}
class SeqPattern extends PatternBase {
    constructor(first, second) {
        super();
        this.first = first;
        this.second = second;
    }
    toRegex() {
        let a = groupForPrecedence(this.first.toRegex(), Precedence.SEQ);
        let b = groupForPrecedence(this.second.toRegex(), Precedence.SEQ);
        return {
            regex: `${a.regex}${b.regex}`,
            precedence: Precedence.SEQ,
            captureGroupCount: a.captureGroupCount + b.captureGroupCount,
            captureNames: mergeMaps(a.captureNames, b.captureNames),
            captureGroups: mergeMaps(a.captureGroups, shiftCaptureGroups(b.captureGroups, a.captureGroupCount)),
            tags: mergeMaps(a.tags, b.tags),
        };
    }
    contentElement() {
        return this;
    }
}
class RepPattern extends PatternBase {
    constructor(element, min, max) {
        super();
        this.element = element;
        this.min = min;
        this.max = max;
    }
    toRegex() {
        let e = groupForPrecedence(this.element.toRegex(), Precedence.REP);
        // Find the nicest operator for the bounds
        let op = (() => {
            if (this.max === undefined) {
                if (this.min == 0)
                    return '*';
                if (this.min == 1)
                    return '+';
                return `{${this.min},}`;
            }
            if (this.min == 0 && this.max == 1)
                return '?';
            if (this.min == this.max)
                return `{${this.min}}`;
            if (this.min == 0)
                return `{,${this.max}}`;
            return `{${this.min},${this.max}}`;
        })();
        return Object.assign(Object.assign({}, e), { regex: `${e.regex}${op}`, precedence: Precedence.REP });
    }
    contentElement() {
        return this;
    }
}
class CapturePattern extends PatternBase {
    constructor(element, name) {
        super();
        this.element = element;
        this.name = name;
    }
    toRegex() {
        let e = this.element.toRegex();
        return {
            regex: `(${e.regex})`,
            precedence: Precedence.GROUP,
            captureGroupCount: e.captureGroupCount + 1,
            captureGroups: extendMap(shiftCaptureGroups(e.captureGroups, 1), [this.contentElement(), 1]),
            captureNames: extendMap(e.captureNames, [this.name, this.contentElement()]),
            tags: e.tags,
        };
    }
    contentElement() {
        return this.element.contentElement();
    }
}
class LookaroundPattern extends PatternBase {
    constructor(element, behind, negate) {
        super();
        this.element = element;
        this.behind = behind;
        this.negate = negate;
    }
    toRegex() {
        let e = this.element.toRegex();
        let op = `${this.behind ? '<' : ''}${this.negate ? '!' : '='}`;
        return Object.assign(Object.assign({}, e), { precedence: Precedence.GROUP, regex: `(?${op}${e.regex})` });
    }
    contentElement() {
        return this.element.contentElement();
    }
}
class TagPattern extends PatternBase {
    constructor(element, tags) {
        super();
        this.element = element;
        this.tags = tags;
    }
    toRegex() {
        if (!(this.element instanceof CapturePattern))
            throw new Error('Tagging requires an underlying capture');
        let e = this.element.toRegex();
        return Object.assign(Object.assign({}, e), { tags: extendMap(e.tags, [this.contentElement(), this.tags]) });
    }
    contentElement() {
        return this.element.contentElement();
    }
}
/**
 * Groups the regex, if the specified precedence is higher, than the one of the regex.
 * @param regex The regex to group.
 * @param prec The precedence to group for.
 */
function groupForPrecedence(regex, prec) {
    if (regex.precedence >= prec)
        return regex;
    return Object.assign(Object.assign({}, regex), { regex: `(?:${regex.regex})`, precedence: Precedence.GROUP });
}
/**
 * Regex precedences.
 */
var Precedence;
(function (Precedence) {
    Precedence.LOWEST = 0;
    Precedence.ALT = 0;
    Precedence.SEQ = 1;
    Precedence.REP = 2;
    Precedence.GROUP = 3;
})(Precedence = exports.Precedence || (exports.Precedence = {}));
/**
 * Escapes a string so when it's used as a regular expression, it literally matches the text.
 * @param text The text to escape.
 * @returns The escaped text, so it can be used in a regex.
 */
function escapeRegex(text) {
    // Source: https://stackoverflow.com/a/6969486
    return text.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'); // $& means the whole matched string
}
/**
 * Computes the statistics of a regex.
 * @param regex The regex to compute the stats for.
 * @returns The statistics for the regex.
 */
function regexStats(regex) {
    // Tracked precedence
    var prec = Precedence.GROUP;
    // Number of capture groups
    var captGroups = 0;
    // Pairwise parenthesis stack
    let stk = [];
    // Character pointer
    var i = 0;
    function updatePrec(p) {
        // We only care, if we are not in a grouping construct
        // which is signaled by an empty stack
        if (stk.length == 0)
            prec = Math.min(prec, p);
    }
    function possibleSequencing() {
        // If this wasn't the first character, we had sequencing
        if (i > 0)
            updatePrec(Precedence.SEQ);
    }
    function charAt(i) {
        if (i < regex.length)
            return regex[i];
        return undefined;
    }
    function top() {
        if (stk.length == 0)
            return undefined;
        return stk[stk.length - 1];
    }
    function tryPop(ch) {
        if (top() != ch)
            return false;
        stk.pop();
        return true;
    }
    while (i < regex.length) {
        // Grouping
        // It's only grouping, if we are not in a character class currently
        if (regex[i] == '(' && top() != ']') {
            possibleSequencing();
            // Skip character
            ++i;
            // We need a closing paren later
            stk.push(')');
            // Check, if there is a '?', signaling a special grouping construct
            // Special constructs are not captured (except named groups, but those are invalid in TextMate)
            if (charAt(i) == '?')
                ++i;
            else
                ++captGroups;
            continue;
        }
        // Character class
        if (regex[i] == '[') {
            possibleSequencing();
            // Skip character
            ++i;
            // We need a closing bracket later
            stk.push(']');
            // Special case, the first character sequence can be a ] or ^] without it closing the bracket
            if (charAt(i) == ']')
                ++i;
            else if (charAt(i) == '^' && charAt(i + 1) == ']')
                i += 2;
            continue;
        }
        // Closing constructs, ')' and `]`
        if (tryPop(regex[i])) {
            ++i;
            continue;
        }
        // Alternation
        if (regex[i] == '|') {
            updatePrec(Precedence.ALT);
            ++i;
            continue;
        }
        // Escaped character
        if (regex[i] == '\\') {
            possibleSequencing();
            i += 2;
            continue;
        }
        // Repetition
        if (i > 0 && "*+?{".indexOf(regex[i]) != -1) {
            updatePrec(Precedence.REP);
            if (regex[i] == '{') {
                // Walk to the '}'
                ++i;
                while (i < regex.length && regex[i] != '}')
                    ++i;
            }
            else {
                ++i;
            }
            continue;
        }
        // Assume raw construct
        possibleSequencing();
        ++i;
    }
    return {
        precedence: prec,
        captureGroupCount: captGroups,
    };
}
exports.regexStats = regexStats;
// Utilities for manipulating maps
function mergeMaps(m1, m2) {
    let result = new Map();
    for (let [k, v] of m1)
        result.set(k, v);
    for (let [k, v] of m2)
        result.set(k, v);
    return result;
}
function extendMap(m, ...vs) {
    return mergeMaps(m, new Map(vs));
}
function shiftCaptureGroups(m, offset) {
    let result = new Map();
    for (let [k, v] of m)
        result.set(k, v + offset);
    return result;
}
