/**
 * Represents a scope that can be highlighted.
 */
export declare class Scope {
    textMateName: string;
    constructor();
    static create(): Scope;
    withTextMateName(tmName: string): Scope;
}
/**
 * Predefined scopes.
 */
export declare namespace Scopes {
    /**
     * Scopes related to comments.
     */
    namespace Comment {
        /**
         * Retrieves the scope most appropriate for the line comment with the given start symbol.
         * @param symbol The starting symbol of the line comment.
         * @returns The best scope for the line comment.
         */
        function line(symbol?: string): Scope;
        /**
         * Block comment.
         */
        const Block: Scope;
        /**
         * Documentation comment.
         */
        const Doc: Scope;
        /**
         * Comment tags, like '@param' or 'FIXME'.
         */
        const Tag: Scope;
        /**
         * Comment tag authors, like TODO(LPeter1997).
         */
        const TagAuthor: Scope;
        /**
         * Punctuation of comments, like '//' or '/*'.
         */
        const Punct: Scope;
    }
    /**
     * Common keywords.
     */
    namespace Keyword {
        /**
         * Constructs a generic keyword scope.
         * @param name The name of the keyword.
         * @returns The constructed scope.
         */
        function other(name?: string): Scope;
        /**
         * Constructs a control keyword scope.
         * @param name The name of the keyword.
         * @returns The constructed scope.
         */
        function control(name?: string): Scope;
        /**
         * Constructs an operator keyword scope.
         * @param name The name of the operator.
         * @returns The constructed scope.
         */
        function operator(name?: string): Scope;
        const Package: Scope;
        const Namespace: Scope;
        const Class: Scope;
        const Struct: Scope;
        const Using: Scope;
        const Import: Scope;
        const Static: Scope;
        const If: Scope;
        const Else: Scope;
        const While: Scope;
        const For: Scope;
        const Continue: Scope;
        const Break: Scope;
        const Return: Scope;
        const Goto: Scope;
    }
    /**
     * Entity names.
     */
    namespace Name {
        /**
         * Constructs a scope for type names.
         * @param name The name of the type.
         * @returns The constructed scope.
         */
        function type(name?: string): Scope;
        /**
         * Constructs a scope for some builtin value, like 'this', 'super' or 'self'.
         * @param name The buildin values name.
         * @returns The constructed scope.
         */
        function builtin(name?: string): Scope;
        const Package: Scope;
        const Namespace: Scope;
        const Class: Scope;
        const Struct: Scope;
        const This: Scope;
        const Super: Scope;
        const Self: Scope;
        const Function: Scope;
        const Attribute: Scope;
        const Base: Scope;
    }
    /**
     * Constructs a scope for modifiers like 'static', 'final', 'abstract', ...
     * @param name The name of the modifier.
     * @returns The constructed scope.
     */
    function modifier(name?: string): Scope;
    /**
     * Constructs a scope for punctuation.
     * @param char The punctuation character(s).
     * @returns The constructed scope.
     */
    function punct(char?: string): Scope;
    /**
     * Literal values.
     */
    namespace Literal {
        /**
         * Constructs a generic literal scope.
         * @param name The name of the literal.
         * @returns The constructed scope.
         */
        function other(name?: string): Scope;
        /**
         * Creates a scope for numeric literals.
         * @param name The name of the numeric literal.
         * @returns The constructed scope.
         */
        function numeric(name?: string): Scope;
        /**
         * Creates a scope for keyword-like literals.
         * @param name The name of the literal.
         * @returns The constructed scope.
         */
        function keyword(name?: string): Scope;
        /**
         * Creates a scope for string literals.
         * @param punct The punctuation character that starts or ends the string.
         */
        function string(punct?: string): Scope;
        const Char: Scope;
        const CharEscape: Scope;
        const RegExp: Scope;
    }
    /**
     * Things related to markup.
     */
    namespace Markup {
        function heading(tag?: string): Scope;
        function snippet(name?: string): Scope;
        const Link: Scope;
        const Underline: Scope;
        const Bold: Scope;
        const Italic: Scope;
        const List: Scope;
        const OrderedList: Scope;
        const UnorderedList: Scope;
        const Quote: Scope;
        const Raw: Scope;
    }
}
