"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Scopes = exports.Scope = void 0;
/**
 * Represents a scope that can be highlighted.
 */
class Scope {
    constructor() {
        this.textMateName = 'undefined';
    }
    static create() {
        return new Scope();
    }
    withTextMateName(tmName) {
        this.textMateName = tmName;
        return this;
    }
}
exports.Scope = Scope;
/**
 * Predefined scopes.
 */
var Scopes;
(function (Scopes) {
    /**
     * Scopes related to comments.
     */
    let Comment;
    (function (Comment) {
        /**
         * Retrieves the scope most appropriate for the line comment with the given start symbol.
         * @param symbol The starting symbol of the line comment.
         * @returns The best scope for the line comment.
         */
        function line(symbol) {
            switch (symbol) {
                case '//':
                    return Scope.create()
                        .withTextMateName('comment.line.double-slash');
                case '--':
                    return Scope.create()
                        .withTextMateName('comment.line.double-dash');
                case '#':
                    return Scope.create()
                        .withTextMateName('comment.line.number-sign');
                case '%':
                    return Scope.create()
                        .withTextMateName('comment.line.percentage');
                default:
                    return Scope.create()
                        .withTextMateName('comment.line');
            }
        }
        Comment.line = line;
        /**
         * Block comment.
         */
        Comment.Block = Scope.create()
            .withTextMateName('comment.block');
        /**
         * Documentation comment.
         */
        Comment.Doc = Scope.create()
            .withTextMateName('comment.block.documentation');
        /**
         * Comment tags, like '@param' or 'FIXME'.
         */
        Comment.Tag = Scope.create()
            .withTextMateName('storage.type.comment-tag');
        /**
         * Comment tag authors, like TODO(LPeter1997).
         */
        Comment.TagAuthor = Scope.create()
            .withTextMateName('storage.type.comment-tag.author');
        /**
         * Punctuation of comments, like '//' or '/*'.
         */
        Comment.Punct = Scope.create()
            .withTextMateName('punctuation.definition.comment');
    })(Comment = Scopes.Comment || (Scopes.Comment = {}));
    /**
     * Common keywords.
     */
    let Keyword;
    (function (Keyword) {
        /**
         * Constructs a generic keyword scope.
         * @param name The name of the keyword.
         * @returns The constructed scope.
         */
        function other(name) {
            return Scope.create().withTextMateName(`keyword.other.${name}`);
        }
        Keyword.other = other;
        /**
         * Constructs a control keyword scope.
         * @param name The name of the keyword.
         * @returns The constructed scope.
         */
        function control(name) {
            return Scope.create().withTextMateName(append('keyword.control', name));
        }
        Keyword.control = control;
        /**
         * Constructs an operator keyword scope.
         * @param name The name of the operator.
         * @returns The constructed scope.
         */
        function operator(name) {
            return Scope.create().withTextMateName(append('keyword.operator', name));
        }
        Keyword.operator = operator;
        Keyword.Package = other('package');
        Keyword.Namespace = other('namespace');
        Keyword.Class = other('class');
        Keyword.Struct = other('struct');
        Keyword.Using = other('using');
        Keyword.Import = other('import');
        Keyword.Static = other('static');
        Keyword.If = control('conditional.if');
        Keyword.Else = control('conditional.else');
        Keyword.While = control('loop.while');
        Keyword.For = control('loop.for');
        Keyword.Continue = control('flow.continue');
        Keyword.Break = control('flow.break');
        Keyword.Return = control('flow.return');
        Keyword.Goto = control('goto');
    })(Keyword = Scopes.Keyword || (Scopes.Keyword = {}));
    /**
     * Entity names.
     */
    let Name;
    (function (Name) {
        /**
         * Constructs a scope for type names.
         * @param name The name of the type.
         * @returns The constructed scope.
         */
        function type(name) {
            return Scope.create().withTextMateName(append('entity.name.type', name));
        }
        Name.type = type;
        /**
         * Constructs a scope for some builtin value, like 'this', 'super' or 'self'.
         * @param name The buildin values name.
         * @returns The constructed scope.
         */
        function builtin(name) {
            return Scope.create().withTextMateName(append('variable.language', name));
        }
        Name.builtin = builtin;
        Name.Package = type('package');
        Name.Namespace = type('namespace');
        Name.Class = type('class');
        Name.Struct = type('struct');
        Name.This = builtin('this');
        Name.Super = builtin('super');
        Name.Self = builtin('self');
        Name.Function = Scope.create()
            .withTextMateName('entity.name.function');
        Name.Attribute = Scope.create()
            .withTextMateName('entity.other.attribute-name');
        Name.Base = Scope.create()
            .withTextMateName('entity.other.inherited-class');
    })(Name = Scopes.Name || (Scopes.Name = {}));
    /**
     * Constructs a scope for modifiers like 'static', 'final', 'abstract', ...
     * @param name The name of the modifier.
     * @returns The constructed scope.
     */
    function modifier(name) {
        return Scope.create().withTextMateName(append('storage.modifier', name));
    }
    Scopes.modifier = modifier;
    /**
     * Constructs a scope for punctuation.
     * @param char The punctuation character(s).
     * @returns The constructed scope.
     */
    function punct(char) {
        switch (char) {
            case '{':
                return Scope.create()
                    .withTextMateName('punctuation.curlybrace.open');
            case '}':
                return Scope.create()
                    .withTextMateName('punctuation.curlybrace.close');
            case '(':
                return Scope.create()
                    .withTextMateName('punctuation.parenthesis.open');
            case ')':
                return Scope.create()
                    .withTextMateName('punctuation.parenthesis.close');
            case '[':
                return Scope.create()
                    .withTextMateName('punctuation.squarebracket.open');
            case ']':
                return Scope.create()
                    .withTextMateName('punctuation.squarebracket.close');
            case ';':
                return Scope.create()
                    .withTextMateName('punctuation.semicolon');
            default:
                return Scope.create()
                    .withTextMateName('punctuation');
        }
    }
    Scopes.punct = punct;
    /**
     * Literal values.
     */
    let Literal;
    (function (Literal) {
        /**
         * Constructs a generic literal scope.
         * @param name The name of the literal.
         * @returns The constructed scope.
         */
        function other(name) {
            return Scope.create().withTextMateName(append('constant.other', name));
        }
        Literal.other = other;
        /**
         * Creates a scope for numeric literals.
         * @param name The name of the numeric literal.
         * @returns The constructed scope.
         */
        function numeric(name) {
            return Scope.create().withTextMateName(append('constant.numeric', name));
        }
        Literal.numeric = numeric;
        /**
         * Creates a scope for keyword-like literals.
         * @param name The name of the literal.
         * @returns The constructed scope.
         */
        function keyword(name) {
            return Scope.create().withTextMateName(append('constant.language', name));
        }
        Literal.keyword = keyword;
        /**
         * Creates a scope for string literals.
         * @param punct The punctuation character that starts or ends the string.
         */
        function string(punct) {
            switch (punct) {
                case '\'':
                    return Scope.create()
                        .withTextMateName('string.quoted.single');
                case '"':
                    return Scope.create()
                        .withTextMateName('string.quoted.double');
                case '"""':
                    return Scope.create()
                        .withTextMateName('string.quoted.triple');
                default:
                    return Scope.create()
                        .withTextMateName('string.quoted.other');
            }
        }
        Literal.string = string;
        Literal.Char = Scope.create()
            .withTextMateName('constant.character');
        Literal.CharEscape = Scope.create()
            .withTextMateName('constant.character.escape');
        Literal.RegExp = Scope.create()
            .withTextMateName('string.regexp');
    })(Literal = Scopes.Literal || (Scopes.Literal = {}));
    /**
     * Things related to markup.
     */
    let Markup;
    (function (Markup) {
        function heading(tag) {
            return Scope.create().withTextMateName(append('markup.heading', tag));
        }
        Markup.heading = heading;
        function snippet(name) {
            return Scope.create().withTextMateName(append('markup.raw', name));
        }
        Markup.snippet = snippet;
        Markup.Link = Scope.create()
            .withTextMateName('markup.underline.link');
        Markup.Underline = Scope.create()
            .withTextMateName('markup.underline');
        Markup.Bold = Scope.create()
            .withTextMateName('markup.bold');
        Markup.Italic = Scope.create()
            .withTextMateName('markup.italic');
        Markup.List = Scope.create()
            .withTextMateName('markup.list');
        Markup.OrderedList = Scope.create()
            .withTextMateName('markup.list.numbered');
        Markup.UnorderedList = Scope.create()
            .withTextMateName('markup.list.unnumbered');
        Markup.Quote = Scope.create()
            .withTextMateName('markup.quote');
        Markup.Raw = Scope.create()
            .withTextMateName('markup.raw');
    })(Markup = Scopes.Markup || (Scopes.Markup = {}));
})(Scopes = exports.Scopes || (exports.Scopes = {}));
function append(s1, s2) {
    if (s2 === undefined)
        return s1;
    return `${s1}.${s2}`;
}
